      subroutine atscat_highres_ephoton_interpolator (ebin_edge_in, 
     . nobins_in, ein, nvbins, maxen, maxchan, matrix,
     . nobins_out, new_matrix)
C Written by Colleen Wilson-Hodge NASA/MSFC 5-November-2008. This code
C interpolates the input (photon) side of the atscat matrix to the user 
C requested input (photon) edges for a particular burst.
C This code was adapted from highres_ephoton_interpolator.f for the case where
C the output side of the matrix was already uncompressed.
      implicit none
C inputs
      integer nobins_in, !number of desired bins on input side of new_matrix
     . nvbins, !number of input bins for input matrix
     . maxen, !maximum input bins for matrix or new_matrix
     . maxchan, !maximum number of output bins for matrix or new_matrix
     . nobins_out !number of output bins for matrix and new_matrix
      real ebin_edge_in(nobins_in+1), !Desired input bin edges
     . ein(*), !original input bin edges
     . matrix(maxen,maxchan) !original matrix
C outputs
       real new_matrix(maxen,maxchan) !interpolated matrix
C local variables
       integer i,j,k, ivfind
       real mu
C start executable code  
       do i=1, maxen
         do j=1, maxchan
	   new_matrix(i,j) = 0.0
	 enddo
       enddo	        
       ivfind=1
       do i=1,nobins_in  
         do j=ivfind, nvbins
           if((ebin_edge_in(i).ge.ein(j)).and.
     .        (ebin_edge_in(i).lt.ein(j+1))) then
             ivfind=j
	     mu=(log(ebin_edge_in(i))-log(ein(ivfind)))/
     .        (log(ein(ivfind+1))-log(ein(ivfind)))
             if ((mu.lt.0.0).and.(mu.gt.-1e-5)) mu=0.0
	     if ((mu.gt.1.0).and.(mu.lt.1.00001)) mu=1.0
	     if ((mu.lt.-1e-5).or.(mu.gt.1.00001)) then
	       print*,'Inside atscat_highres_ephoton_interpolator:',
     .          ' Unexpected value for mu: ',mu
               stop
	     endif  
             do k=1,nobins_out
	       new_matrix(i,k)=(matrix(ivfind,k)*(1-mu)+
     .	        matrix(ivfind+1,k)*mu)
             enddo
	   endif
	 enddo  
       enddo	   
       return
       end	      
      
