      subroutine echan_integrator (nobins_in, maxen, maxchan, 
     $ diff_matrix, edif_edge_lo, edif_edge_hi, nhbins,
     $ ebin_edge_out, nobins_out, debug, binned_matrix)
     
C Adapted by Colleen Wilson-Hodge from BATSE code written by Geoff Pendleton 2007-Jul-31    
C Removed interpolation over input (photon) energies - CAWH 2008-Nov-4

C  Cleanup and make more readable by Michael S. Briggs on 2009 August 2.
C  Use guidance of ftncheck and spag (of PlusFort) to remove declarations of unused variables.
C  Add a lot more whitespace (spaces within statements, and between lines) to make more readable.
C  I increased the line length up to 132 and merged continuations lines into fewer lines, typically one,
C  for improved readability.  This is technically an extension, but, should be supported with almost
C  any compiler using options/flags.   With gfortran, use "-ffixed-form -ffixed-line-length-none".
C  I realized that some of Geoff's comments appear AFTER the code that they are describing --
C  when I thought I understood the code & comment, I relocated the comment.  There are probably
C  still comments that appear after the code that they describe.

      implicit none
       
c inputs: 
      integer nobins_in,nobins_out,nhbins,maxen,maxchan, debug
      
      real  diff_matrix (maxen,maxchan), edif_edge_lo (maxen,maxchan), edif_edge_hi (maxen,maxchan), ebin_edge_out (nobins_out+1)

c outputs:
      real binned_matrix (maxen,maxchan)

c local variables:
      integer jcdif, ivh, ihlow, ihhigh, ihbin, ihlfind, ihfind, ivhsum, ihover, nhpoints, icbin, icdif, i, j
      
      real diff_matrix_vec (nhbins), edif_edgeh (nhbins+1), edif_cent (nhbins), row_tot (nobins_out+1),
     $   hlow, hhigh, hwide, hchunk, euse, row_entry
     
c additional local variables for cosine interpolation
      real pi, mu, mu2
      
C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

      pi = acos(-1.0)
      
      if(debug.gt.1) then 
         print*,'nobins_in,nobins_out,nhbins: ', nobins_in,nobins_out,nhbins
         print*,'ebin_edge_out = ', (ebin_edge_out(i), i=1,nobins_out+1)
      endif
      
      do i=1, nobins_out + 1
         row_tot(i)=0.
      enddo
      
      do i=1, nobins_in
         do j=1, nobins_out
            binned_matrix(i,j)=0.
         enddo
      enddo

c     loop over input (photon) energies  
      do JCDIF=1, nobins_in 
      
c        Extract row and divide by bin width to make cm^2/keV units.  
         do IVH=1, nhbins
         
            diff_matrix_vec (ivh) = diff_matrix (jcdif,ivh) / (edif_edge_hi(jcdif,ivh) - edif_edge_lo(jcdif,ivh))
            EDIF_EDGEH(ivh) = edif_edge_hi(JCDIF,ivh)
            EDIF_CENT(ivh) = (edif_edge_lo(JCDIF,ivh)+edif_edge_hi(JCDIF,ivh))/2.
     
            if (debug.gt.1) print*,ivh,diff_matrix_vec(ivh)
         enddo !DO IVH=1,nhbins
        
         EDIF_EDGEH (nhbins+1) = edif_edge_hi(JCDIF,nhbins) + (edif_edge_hi(jcdif,nhbins) - edif_edge_hi(jcdif,nhbins-1))
     
         if (debug.gt.1) then
            print*,'JCDIF: ', jcdif
            print*,'Edif_cent=',(edif_cent(i),i=1,nhbins)
            print*,'Edif_edgeh=',(edif_edgeh(i),i=1,nhbins+1)
         endif 
        
         IHLOW=0
         IHHIGH=0
   
C READY FOR  THE HORIZONTAL SUMMATION
c ---------------- HORIZONTAL LOOP -----------------

         do IHBIN=1, NOBINS_OUT ! loop over output bins

            HLOW=EBIN_EDGE_OUT(IHBIN)
            HHIGH=EBIN_EDGE_OUT(IHBIN+1)
            HWIDE=HHIGH-HLOW
            
            IF (IHLOW .EQ. 0) THEN 
            
c              next we identify the compressed matrix horizontal bin indices
c              HLOW and IHHIGH. THIS IS THE FIRST HORIZONTAL BIN SO SEARCH FOR IHLOW
               do IHLFIND=1, nhbins - 1 
                  IF ( (HLOW .GT. EDIF_CENT (IHLFIND)) .AND. (HLOW .LE. EDIF_CENT (IHLFIND+1)) ) THEN 
                     IHLOW=IHLFIND
                     goto 150
                  ENDIF !IF( (HLOW.GT.EDIF_CENT(IHLFIND)).AND.
               enddo ! DO IHLFIND=1,nhbins-1
 150           continue    
            ENDIF !IF(IHLOW.EQ.0) THEN   


c  IF IHLOW IS STILL ZERO AFTER THIS TREMENDOUS EFFORT TO 
c NON ZERO IT THEN HLOW LT EDIF_CENT(1). ASSUME IHHIGH ON SCALE
c next we are going to break the search into three sections for the 
c horizontal bin calculation. The first section is for binned matrix 
c elements that overflow the bottom of the compressed horizontal matrix
c data ( delineated by (C111111111111111111111111111) commnets).
c the second section is for overflowing the top end of the horizontal
c matrix ( delineated by (C22222222222222222222222222) comments)
c the third section is for horizontal rows contained by the comressed matrix energies.
c ( delineated by (C333333333333333333333333333333) comments) 

c secton 1 for overflowing the bottom of the hrizontal matrix
c 111111111111111111111111111111111111111111111111111111111111111
c 111111111111111111111111111111111111111111111111111111111111111

            IF (HLOW .LE. EDIF_CENT (1)) THEN 
               IF (HHIGH .GT. EDIF_CENT (1)) THEN 
               
c LOCATE IHHIGH
                  do IHFIND=1, nhbins - 1 
                     IF ( (HHIGH .GT. EDIF_CENT (IHFIND)) .AND. (HHIGH .LE. EDIF_CENT (IHFIND+1)) )  IHHIGH=IHFIND
                  enddo !DO IHFIND=1,nhbins-1
              
c                 if(HHIGH.GT.EDIF_CENT(nhbins) ) IHHIGH=nhbins-1
                  NHPOINTS=IHHIGH+2
                  HCHUNK=HWIDE/FLOAT(NHPOINTS-1)
              
                  do ICBIN=1, NHPOINTS
               
                     EUSE = HLOW + HCHUNK * FLOAT (ICBIN-1)
                  
                     IF (EUSE .LE. EDIF_CENT(1)) THEN
c                       the  *EUSE/EDIF_CENT(1) results in tapering to 0.0 below lowest horizontal compressed point                   
                        ROW_ENTRY = DIFF_MATRIX_VEC(1) * EUSE / EDIF_CENT(1)
                        GOTO 1500
                     ENDIF ! IF(EUSE.LE.EDIF_CENT(1)) THEN
                  
                     IF (EUSE .GT. EDIF_CENT(1)) THEN 
                   
                        do ICDIF=1, IHHIGH
                           IF ( (EUSE .GT. EDIF_CENT (ICDIF))  .AND. (EUSE.LE.EDIF_CENT(ICDIF+1))) THEN 
c                             standard interpolation betweeen two points in horizontal energy                        
                              ROW_ENTRY = DIFF_MATRIX_VEC (ICDIF) + ( DIFF_MATRIX_VEC (ICDIF+1) - DIFF_MATRIX_VEC (ICDIF) ) *
     $                            ( EUSE-EDIF_CENT (ICDIF) ) / ( EDIF_CENT (ICDIF+1) - EDIF_CENT (ICDIF) )
                              GOTO 1500
                           ENDIF  ! IF( (EUSE.GT.EDIF_CENT(ICDIF)) .AND.
                        ENDDO !DO ICDIF=1,IHHIGH
                      
                     ENDIF ! IF(EUSE.GT.EDIF_CENT(1)) 
 1500                continue

                     ROW_TOT(IHBIN) = ROW_ENTRY+ROW_TOT(IHBIN) ! sum up horizontal 
c                    contributions each with the same weighting defined by the width HCHUNK
                     ROW_ENTRY=0.0 ! reset this each time
                  ENDDO ! END OF ICBIN LOOP
               
c                 HWIDE = horizontal bin width 
c                 NHPOINTS = # of samples used
c                 convert from counts/(unit energy) to counts/bin               
                  ROW_TOT (IHBIN) = ROW_TOT (IHBIN) * HWIDE / float (NHPOINTS)
               
               ENDIF ! HHIGH GT EDIF_CENT(1)

c              here just load this one element extrapolated downward from EDIF_CENT(1)            
               IF (HHIGH .LE. EDIF_CENT (1) )  ROW_TOT (IHBIN) = DIFF_MATRIX_VEC (1) * ( (HLOW+HHIGH)/2.) / EDIF_CENT(1) * HWIDE

            ENDIF ! END OF LOW HLOW LOOP
          
c end secton 1 for overflowing the bottom of the hrizontal matrix
c 111111111111111111111111111111111111111111111111111111111111111
c 111111111111111111111111111111111111111111111111111111111111111


c section 2 for overflowwing the top of the horizontal compressed matrix
c 22222222222222222222222222222222222222222222222222222222222222222
c 22222222222222222222222222222222222222222222222222222222222222222
c IHLOW= LAST LOWER EDIF_CENT INDEX: BELOW PREVIOUS EDGE

            IF (IHLOW .GE. nhbins) THEN 
               IF (HLOW .GT. EDIF_EDGEH (nhbins+1)) THEN 
                  ROW_TOT (IHBIN) = -1.0 ! not used in sum below (check 700 continue statment)
                  IHOVER = IHBIN
                  GOTO 600
               ENDIF ! IF(HLOW.GT.EDIF_EDGEH(65))
             
               IF (HLOW .LE. EDIF_EDGEH (nhbins+1)) THEN 
c                 COULD STILL BE BELOW THIS EDGE
                  IF (HHIGH .LE. EDIF_EDGEH (nhbins+1)) THEN 
c                    BOTH BIN EDGES GREATER THAN EDIF_CENT(64) AND LESS THAN EDIF_EDGEH(65)
c                    interpolation to bin center with HWIDE providing counts/bin correction
                     ROW_TOT (IHBIN) = DIFF_MATRIX_VEC (nhbins) * (EDIF_EDGEH(nhbins+1) - (HLOW+HHIGH) / 2.0) / 
     $                   (EDIF_EDGEH (nhbins+1) - EDIF_CENT (nhbins)) * HWIDE
                     GOTO 600
                  ENDIF !IF(HHIGH.LE.EDIF_EDGEH(65)) 
              
                  IF (HHIGH .GT. EDIF_EDGEH (nhbins+1)) THEN 
c                    ONE EDGE LESS, ONE EDGE GREATER
c                    interpolation with truncated bin width since the bin exstends outside the horizontal range
                     ROW_TOT (IHBIN) = ((EDIF_EDGEH (nhbins+1) - HLOW)**2 ) * DIFF_MATRIX_VEC (nhbins) /
     $                   (2. * (EDIF_EDGEH (nhbins+1) - EDIF_CENT (nhbins)))
                     GOTO 600
                  ENDIF ! IF(HHIGH.GT.EDIF_EDGEH(65))
               ENDIF ! IF(HHIGH.GT.EDIF_EDGEH(65))
            ENDIF  ! IF(IHLOW.GE.64) THEN

c end section 2 for overflowwing the top of the horizontal compressed matrix
c 22222222222222222222222222222222222222222222222222222222222222222
c 22222222222222222222222222222222222222222222222222222222222222222

c section 3 for horizontal elements contained by the compressed matrix
c 33333333333333333333333333333333333333333333333333333333333333333
c 33333333333333333333333333333333333333333333333333333333333333333


            IF ( (IHLOW .LT. nhbins) .AND. (IHLOW .GE. 1) ) THEN 
               IF (HHIGH .GT. EDIF_EDGEH (nhbins+1)) THEN 
                  HWIDE = EDIF_EDGEH (nhbins+1) - HLOW ! total width adjusted for active response range
                  NHPOINTS = nhbins - IHLOW + 2
                  HCHUNK = HWIDE / FLOAT (NHPOINTS-1)
               
                  do ICBIN=1, NHPOINTS 
                     EUSE = HLOW + HCHUNK * FLOAT (ICBIN-1)
                     do ICDIF=IHLOW, nhbins - 1 
                        IF ( (EUSE .GT. EDIF_CENT (ICDIF) ) .AND. (EUSE .LE. EDIF_CENT (ICDIF+1)) ) THEN 
                           mu = (EUSE-EDIF_CENT(ICDIF)) / (EDIF_CENT(ICDIF+1)-EDIF_CENT(ICDIF))
                           mu2 = (1-cos(mu*pi)) / 2.0
                           row_entry = DIFF_MATRIX_VEC (ICDIF) + (DIFF_MATRIX_VEC (ICDIF+1) - DIFF_MATRIX_VEC(ICDIF)) * mu2
                           goto 400
                        ENDIF ! IF( (EUSE.GT.EDIF_CENT(ICDIF)) .AND.
                     ENDDO !DO ICDIF=IHLOW,63   

 400                 continue
 
                     ROW_TOT (IHBIN) = ROW_ENTRY + ROW_TOT (IHBIN)     
                     ROW_ENTRY = 0.0
             
                  enddo ! DO ICBIN=1,NHPOINTS
              
                  ROW_TOT (IHBIN) = ROW_TOT (IHBIN) * HWIDE / float (NHPOINTS) !cawh 12-16-2005
                  IHLOW=nhbins
              
               ENDIF ! IF(HHIGH.GT.EDIF_EDGEH(65)  

               IF (HHIGH .LE. EDIF_EDGEH (nhbins+1)) THEN 
c                 HERE WE ARE STILL INSIDE THE COMPRESSED ROW
c                 SO THIS IS THE BASIC SUMATION AND HWIDE IS LEFT UNVIOLATED

                  do IHFIND=IHLOW, nhbins-1 
c                    SEARCH TEST
                     IF ( (HHIGH .GT. EDIF_CENT(IHFIND) ) .AND. (HHIGH .LE. EDIF_CENT(IHFIND+1) ) )  IHHIGH=IHFIND                 
                  enddo !DO IHFIND=IHLOW,63
            
                  IF (HHIGH .GT. EDIF_CENT (nhbins)) IHHIGH=nhbins
                  NHPOINTS = IHHIGH-IHLOW+2
                  if (NHPOINTS .lt. 9) NHPOINTS = 9
                  HCHUNK = HWIDE / FLOAT (NHPOINTS-1)
                  do ICBIN=1, NHPOINTS
                     EUSE = HLOW + HCHUNK * FLOAT (ICBIN-1)             
                     do ICDIF=IHLOW, IHHIGH 
               
                        IF (ICDIF .LE. nhbins-1) THEN 
                           IF ((EUSE .GT. EDIF_CENT (ICDIF)) .AND. (EUSE .LE. EDIF_CENT (ICDIF+1)) ) THEN 

c                             note: this samples to EDIF_CENT(IHHIGH+!)
c                             standard interpolation
                              ROW_ENTRY = DIFF_MATRIX_VEC (ICDIF) + (DIFF_MATRIX_VEC (ICDIF+1) - DIFF_MATRIX_VEC (ICDIF)) *
     $                            (EUSE-EDIF_CENT (ICDIF)) / (EDIF_CENT (ICDIF+1) - EDIF_CENT (ICDIF))
                              goto 500
                           ENDIF !IF( (EUSE.GT.EDIF_CENT(ICDIF)) .AND. 
                        ENDIF ! IF(ICDIF.LE.63) 
                  
                        IF (ICDIF .GT. nhbins-1) THEN 
c                          extrapolation                  
                           ROW_ENTRY = DIFF_MATRIX_VEC(ICDIF) * (HHIGH-EDIF_CENT(nhbins)) / 
     $                             (EDIF_CENT (nhbins) - EDIF_CENT (nhbins-1))
                           GOTO 500
                        ENDIF ! IF(ICDIF.GT.63)                    
                     ENDDO ! DO ICDIF=IHLOW,IHHIGH
               
               
 500                 continue
                     ROW_TOT (IHBIN) = ROW_ENTRY + ROW_TOT (IHBIN)
                     ROW_ENTRY = 0.0
               
                  ENDdo ! DO ICBIN=1,NHPOINTS
            
                  ROW_TOT (IHBIN) = ROW_TOT (IHBIN) * HWIDE / FLOAT (NHPOINTS) !cawh 12-16-2005
                  IHLOW = IHHIGH
               ENDIF ! IF(HHIGH.LE.EDIF_EDGEH(65)) 
            ENDIF  ! IF((IHLOW.LT.64).AND.(IHLOW.GE.1)) 

c end section 3 for horizontal elements contained by the compressed matrix
c 33333333333333333333333333333333333333333333333333333333333333333
c 33333333333333333333333333333333333333333333333333333333333333333

 600        continue
            IF (ROW_TOT (IHBIN) .EQ. -1.0) GOTO 700
            IF (IHBIN .EQ. NOBINS_OUT)  IHOVER = NOBINS_OUT + 1
         enddo !DO IHBIN=1,NOBINS_OUT loop over output bins

 700     continue

c NOW IN VERTICAL BIT ONLY GOTO IHOVER which can be lower than NOBINS_OUT+1
c REMEMBER TO RESET ROW_TOT

         do IVHSUM=1, IHOVER - 1
            BINNED_MATRIX (JCDIF,IVHSUM) = BINNED_MATRIX (JCDIF,IVHSUM) + ROW_TOT (IVHSUM)
            ROW_TOT (IVHSUM)=0.0 ! empty for next round
         ENDDO 
         
         ROW_TOT(IHOVER)=0.0
         
      enddo ! END OF loop over photon energies
      
      
      end       

   
