      subroutine geo_to_space(theta_u, phi_u, gx, gy, gz, 
     . dirx, diry, dirz, az, el)
C This subroutine computes the current scattering direction in spacecraft 
C coordinates.
C Inputs:
C theta_u - Polar angle for the atmospheric scattering flux coming up
C           from the atmosphere towards the spacecraft. (degrees).
C phi_u - azimuthal angle in scattering database coordinates. The zero point of 
C         phi_u is easiest to understand for flux skimming the atmosphere at
C         the horizon. Zero degrees represents photons moving in a direction
C         close to the incident photon direction. (degrees).
C gx, gy, gz - direction cosines to convert from the scattering database to
C              spacecraft coordinates
C Outputs: 
C dirx, diry, dirz: The scattering direction unit vector in spacecraft
C coordinates.
C az, el -  the scattering direction in spacecraft az, el
      implicit none
      real theta_u, phi_u, gx(3), gy(3), gz(3), dirx, diry, dirz,
     . az, el
      real r, xg, yg, zg
      real*8 dtr
      dtr=acos(-1.d0)/180.d0
      xg=sin(theta_u*dtr)*cos(phi_u*dtr)
      yg=sin(theta_u*dtr)*sin(phi_u*dtr)
      zg=cos(theta_u*dtr)

      dirx=xg*gx(1)+yg*gy(1)+zg*gz(1)
      diry=xg*gx(2)+yg*gy(2)+zg*gz(2)
      dirz=xg*gx(3)+yg*gy(3)+zg*gz(3)

      r = sqrt(dirx*dirx+diry*diry+dirz*dirz)
      dirx=dirx/r
      diry=diry/r
      dirz=dirz/r
      
      az = atan2(diry, dirx)/dtr
      if (az.lt.0.0) az = az +360.0
      el = 90.0 - acos(dirz)/dtr
      end
      	     
