C Written by Colleen A. Wilson-Hodge NASA/MSFC 2007-Jul-31
C Changes 2008-March-06: added tripath to write_triangulation_table and 
C  read_triangulation_table. All routines now use the full name = path+filename
C  when opening files.
C The following subroutines are:
C write_triangulation_hdr0 - writes the primary header for a new file storing
C   the triangulation info
C write_triangulation_table - writes a binary table extension to the fits file
C   containing the LIST, LPR, and LEND arrays created by TRMESH and used by
C   TRFIND in the Renka code (algs 772-3). Also included are the x,y,z and 
C   azimuth zenith positions for all of the grid points used in the 
C   triangulation. Both positions are included because TRMESH requires the 
C   x,y,z position and the files are labeled with the az/zenith positions. 
C read_triangulation_table - reads the LIST, LPTR, LEND arrays needed by 
C   TRMESH along with the x,y,z and az/zenith positions of the gridpoints.
C printerror - fitsio routine to print errors from fitsio routines.
C
      subroutine write_triangulation_hdr0 (filename, tripath)

C Create the primary header, very simple for now
      integer status,unit,blocksize,bitpix,naxis,naxes,testlen
      character filename*80,tripath*80,fullfilename*160
      logical simple,extend
C  The STATUS parameter must be initialized before using FITSIO.  A
C  positive value of STATUS is returned whenever a serious error occurs.
C  FITSIO uses an `inherited status' convention, which means that if a
C  subroutine is called with a positive input value of STATUS, then the
C  subroutine will exit immediately, preserving the status value. For 
C  simplicity, this program only checks the status value at the end of 
C  the program, but it is usually better practice to check the status 
C  value more frequently.

      status=0

C  Get an unused Logical Unit Number to use to open the FITS file.
C  This routine is not required;  programmers can choose any unused
C  unit number to open the file.
      call ftgiou(unit,status)

C  Create the new empty FITS file.  The blocksize parameter is a
C  historical artifact and the value is ignored by FITSIO.
      blocksize=0
      testlen=len_trim(tripath)
      fullfilename=tripath(1:testlen)//filename         
      call ftinit(unit,fullfilename,blocksize,status)

C  Initialize parameters about the FITS image.
C  BITPIX = 16 means that the image pixels will consist of 16-bit
C  integers.  The size of the image is given by the NAXES values. 
C  The EXTEND = TRUE parameter indicates that the FITS file
C  may contain extensions following the primary array.
      simple=.true.
      bitpix=8
      naxis=0
      extend=.true.
C  Write the required header keywords to the file
      call ftphpr(unit,simple,bitpix,naxis,naxes,0,1,extend,status)
C  The FITS file must always be closed before exiting the program. 
C  Any unit numbers allocated with FTGIOU must be freed with FTFIOU.
C Write the creation date keyword to the file
      call ftpdat(unit, status)
      call ftpcom(unit, tripath//
     . 'Path to response database',status)
      call ftclos(unit, status)
      call ftfiou(unit, status)

      end
C****************************************************************************************      
      subroutine write_triangulation_table (tripath,filename,list,lptr,
     .            lend, x,y,z,az,zenith,milliaz,millizen,npoints)
      integer status,unit,readwrite,blocksize,hdutype,tfields,nrows,
     .         nelem,colnum,frow,felem,varidat,npoints,nlist
      integer LIST(*),LPTR(*),LEND(*),milliaz(*),millizen(*),i, testlen
      real x(*),y(*),z(*),az(*),zenith(*)
      character*80 filename, tripath
      character*160 fullfilename

      character*16 ttype(10), tform(10), tunit(10),extname,string
      data ttype/'LIST','LPTR','LEND','X','Y','Z','Azimuth','Zenith',
     . 'milliaz','millizen'/
c      data tform/'1000J', '1000J', '162J', '162E', '162E','162E',
c     .           '162E','162E'/
      status=0
C  Define tform array for number of points that you have. The LIST and LTPR
C  arrays have 6 points for every triangulation grid point.
      nlist = 6*npoints
      write (STRING,'(I5A1)') nlist,'J'
      do i = 1,2
        tform(i)=string
      enddo
      write (STRING,'(I5A1)') npoints,'J'
      tform(3) = string
      write (STRING,'(I5A1)') npoints,'E'
      do i = 4, 8 
        tform(i)=string
      enddo
      do i = 9,10
        write (string,'(I5A1)') npoints,'J'
        tform(i)=string
        tunit(i)='mdeg'
      enddo	
      do i=1,3 
         tunit(i)=' '
      enddo
      do i=4,6
         tunit(i)='radians'
      enddo
      do i=7,8
         tunit(i)='deg'
      enddo	 
C  Get an unused Logical Unit Number to use to open the FITS file.
C  This routine is not required;  programmers can choose any unused
C  unit number to open the file.
      call ftgiou(unit,status)

      testlen=len_trim(tripath)
      fullfilename=tripath(1:testlen)//filename         

C  Open the FITS file, with write access.
      readwrite=1
      call ftopen(unit,fullfilename,readwrite,blocksize,status)

C  Move to the last (1st) HDU in the file .
      call ftmahd(unit,1,hdutype,status)

C  Append/create a new empty HDU onto the end of the file and move to it.
      call ftcrhd(unit,status)
      tfields = 10
      nrows = 1
      extname = 'TRIANGULATION'
      varidat=0
      
C  FTPHBN writes all the required header keywords which define the
C  structure of the binary table. NROWS and TFIELDS gives the number of
C  rows and columns in the table, and the TTYPE, TFORM, and TUNIT arrays
C  give the column name, format, and units, respectively of each column.
      call ftphbn(unit,nrows,tfields,ttype,tform,tunit,
     &            extname,varidat,status)
      frow=1
      felem=1
      colnum=1
      nelem=nlist
      call ftpclj(unit,colnum,frow,felem,nelem,list,status) 
      colnum=2
      call ftpclj(unit,colnum,frow,felem,nelem,lptr,status) 
      colnum=3
      nelem=npoints
       call ftpclj(unit,colnum,frow,felem,nelem,lend,status) 
      colnum=4
       call ftpcle(unit,colnum,frow,felem,nelem,x,status) 
      colnum=5
       call ftpcle(unit,colnum,frow,felem,nelem,y,status) 
      colnum=6
       call ftpcle(unit,colnum,frow,felem,nelem,z,status) 
      colnum=7
       call ftpcle(unit,colnum,frow,felem,nelem,az,status) 
      colnum=8
       call ftpcle(unit,colnum,frow,felem,nelem,zenith,status) 
      colnum=9
       call ftpclj(unit,colnum,frow,felem,nelem,milliaz,status) 
      colnum=10
       call ftpclj(unit,colnum,frow,felem,nelem,millizen,status) 
     
C compute and write the DATASUM and CHECKSUM keywords to the primary header
       call ftpcks(unit, status)       
C  The FITS file must always be closed before exiting the program. 
C  Any unit numbers allocated with FTGIOU must be freed with FTFIOU.
      call ftclos(unit, status)
      call ftfiou(unit, status)

C  Check for any error, and if so print out error messages.
C  The PRINTERROR subroutine is listed near the end of this file.
      if (status .gt. 0)call printerror(status)
      end
C**************************************************************************
       subroutine read_triangulation_table(tripath,filename,list,lptr,
     .            lend, x,y,z,az,zenith,milliaz,millizen,npoints)

C  Read and print data values from an ASCII or binary table
C  This example reads and prints out all the data in the ASCII and
C  the binary tables that were previously created by WRITEASCII and
C  WRITEBINTABLE.  Note that the exact same FITSIO routines are
C  used to read both types of tables.

      integer status,unit,readwrite,blocksize,hdutype
      integer felem,nelems,nullj,nfound,irow,colnum,testlen
      real nulle
      integer LIST(*),LPTR(*),LEND(*),i,npoints,nlist
      integer milliaz(*),millizen(*)
      real x(*),y(*),z(*),az(*),zenith(*)
      character filename*80, tripath*80, fullfilename*160
      logical anynull
      integer datacode, repeat, width
C  The STATUS parameter must always be initialized.
      status=0

C  Get an unused Logical Unit Number to use to open the FITS file.
      call ftgiou(unit,status)
      testlen=len_trim(tripath)
      fullfilename=tripath(1:testlen)//filename         

C  Open the FITS file previously created by WRITEIMAGE
      readwrite=0
      call ftopen(unit,fullfilename,readwrite,blocksize,status)
C  Move to the next extension
      call ftmahd(unit,2,hdutype,status)
C read in the data one column at a time.
      colnum = 1
      irow = 1 
      felem = 1
      nullj = 0
C Get the number of elements in the 1st column (LIST) assume
C LPTR (the 2nd column) has the same # of elements.      
      call ftgtcl(unit, colnum, datacode, nlist, width, status)
C Read the first column LIST
      call ftgcvj(unit, colnum, irow, felem, nlist, nullj, list,
     .            anynull, status)  
C Read the second column LPTR
      colnum = 2
      call ftgcvj(unit, colnum, irow, felem, nlist, nullj, lptr,
     .            anynull, status)  
C read the number of elements in the 3rd column X. Assume that columns
C 4-8 y,z, az, el have the same number of elements.
      colnum = 3
      call ftgtcl(unit, colnum, datacode, npoints, width, status)
      call ftgcvj(unit, colnum, irow, felem, npoints, nullj, lend,
     .            anynull, status)  
      colnum = 4
      call ftgcve(unit, colnum, irow, felem, npoints, nulle, x,
     .            anynull, status)  
      colnum = 5
      call ftgcve(unit, colnum, irow, felem, npoints, nulle, y,
     .            anynull, status)  
      colnum = 6
      call ftgcve(unit, colnum, irow, felem, npoints, nulle, z,
     .            anynull, status)  
      colnum = 7
      call ftgcve(unit, colnum, irow, felem, npoints, nulle, az,
     .            anynull, status)  
      colnum = 8
      call ftgcve(unit, colnum, irow, felem, npoints, nulle,
     .            zenith,anynull, status)  
      colnum = 9
      call ftgcvj(unit, colnum, irow, felem, npoints, nullj, milliaz,
     .            anynull, status)  
      colnum = 10
      call ftgcvj(unit, colnum, irow, felem, npoints, nullj, millizen,
     .            anynull, status)  

C  The FITS file must always be closed before exiting the program. 
C  Any unit numbers allocated with FTGIOU must be freed with FTFIOU.
      call ftclos(unit, status)
      call ftfiou(unit, status)

C  Check for any error, and if so print out error messages.
C  The PRINTERROR subroutine is listed near the end of this file.
      if (status .gt. 0)call printerror(status)
      end
	    

      
     
C *************************************************************************
      subroutine printerror(status)

C  This subroutine prints out the descriptive text corresponding to the
C  error status value and prints out the contents of the internal
C  error message stack generated by FITSIO whenever an error occurs.

      integer status
      character errtext*30,errmessage*80

C  Check if status is OK (no error); if so, simply return
      if (status .le. 0)return

C  The FTGERR subroutine returns a descriptive 30-character text string that
C  corresponds to the integer error status number.  A complete list of all
C  the error numbers can be found in the back of the FITSIO User's Guide.
      call ftgerr(status,errtext)
      print *,'FITSIO Error Status =',status,': ',errtext

C  FITSIO usually generates an internal stack of error messages whenever
C  an error occurs.  These messages provide much more information on the
C  cause of the problem than can be provided by the single integer error
C  status value.  The FTGMSG subroutine retrieves the oldest message from
C  the stack and shifts any remaining messages on the stack down one
C  position.  FTGMSG is called repeatedly until a blank message is
C  returned, which indicates that the stack is empty.  Each error message
C  may be up to 80 characters in length.  Another subroutine, called
C  FTCMSG, is available to simply clear the whole error message stack in
C  cases where one is not interested in the contents.
      call ftgmsg(errmessage)
      do while (errmessage .ne. ' ')
          print *,errmessage
          call ftgmsg(errmessage)
      end do
      end

      

      

